@@ -24,6 +24,7 @@
 
 #include "qtractorTrack.h"
 
+
 // Forward declarations.
 class qtractorBus;
 class qtractorSessionCursor;
@@ -43,6 +44,7 @@ public:
 	// Constructor.
 	qtractorEngine(qtractorSession *pSession,
 		qtractorTrack::TrackType syncType);
+
 	// Destructor.
 	virtual ~qtractorEngine();
 
@@ -98,6 +100,9 @@ public:
 	virtual bool loadElement(qtractorDocument *pDocument, QDomElement *pElement) = 0;
 	virtual bool saveElement(qtractorDocument *pDocument, QDomElement *pElement) const = 0;
 
+	// Clear/reset all pending connections.
+	void clearConnects();
+
 protected:
 
 	// Derived classes must set on this...
@@ -136,11 +141,11 @@ class qtractorBus : public qtractorList<qtractorBus>::Link
 public:
 
 	// Bus operation mode bit-flags.
-	enum BusMode { None = 0, Input = 1, Output = 2, Duplex = 3 };
+	enum BusMode { None = 0, Input = 1, Output = 2, Duplex = 3, Ex = 4 };
 
 	// Constructor.
-	qtractorBus(qtractorEngine *pEngine, const QString& sBusName,
-		BusMode busMode = Duplex, bool bMonitor = false);
+	qtractorBus(qtractorEngine *pEngine,
+		const QString& sBusName, BusMode busMode, bool bMonitor = false);
 
 	// Destructor.
 	virtual ~qtractorBus();
