@@ -184,6 +184,7 @@ public slots:
 
 	void clipNew();
 	void clipEdit();
+	void clipUnlink();
 	void clipSplit();
 	void clipMerge();
 	void clipNormalize();
@@ -199,6 +200,13 @@ public slots:
 	void clipToolsResize();
 	void clipToolsRescale();
 	void clipToolsTimeshift();
+	void clipTakeSelect(QAction *pAction);
+	void clipTakeFirst();
+	void clipTakePrev();
+	void clipTakeNext();
+	void clipTakeLast();
+	void clipTakeReset();
+	void clipTakeRange();
 
 	void viewMenubar(bool bOn);
 	void viewStatusbar(bool bOn);
@@ -274,6 +282,8 @@ public slots:
 	void updateCurveMenu();
 	void updateCurveModeMenu();
 	void updateClipMenu();
+	void updateTakeMenu();
+	void updateTakeSelectMenu();
 	void updateExportMenu();
 	void updateZoomMenu();
 	void updateSnapMenu();
