@@ -124,6 +124,48 @@ uint qHash ( const qtractorMidiClip::Key& key )
 qtractorMidiClip::Hash qtractorMidiClip::g_hashTable;
 
 
+//----------------------------------------------------------------------
+// class qtractorMidiClip::FileKey -- MIDI file hash key.
+//
+class qtractorMidiClip::FileKey
+{
+public:
+
+	// Constructor.
+	FileKey(qtractorMidiClip::Key *pKey) :
+		m_sFilename(pKey->filename()),
+		m_iTrackChannel(pKey->trackChannel()) {}
+
+	// Key accessors.
+	const QString& filename() const
+		{ return m_sFilename; }
+	unsigned short trackChannel() const
+		{ return m_iTrackChannel; }
+
+	// Match descriminator.
+	bool operator== (const FileKey& other) const
+	{
+		return m_sFilename     == other.filename()
+			&& m_iTrackChannel == other.trackChannel();
+	}
+
+private:
+
+	// Interesting variables.
+	QString        m_sFilename;
+	unsigned short m_iTrackChannel;
+};
+
+
+uint qHash ( const qtractorMidiClip::FileKey& key )
+{
+	return qHash(key.filename()) ^ qHash(key.trackChannel());
+}
+
+
+qtractorMidiClip::FileHash qtractorMidiClip::g_hashFiles;
+
+
 //----------------------------------------------------------------------
 // class qtractorMidiClip -- MIDI sequence clip.
 //
@@ -231,7 +273,7 @@ bool qtractorMidiClip::createMidiFile (
 	m_pData->attach(this);
 
 	// Right on then...
-	g_hashTable.insert(*m_pKey, m_pData);
+	insertHashKey();
 
 	qtractorMidiSequence *pSeq = m_pData->sequence();
 
@@ -451,8 +493,8 @@ bool qtractorMidiClip::openMidiFile (
 
 	// Something might have changed...
 	if (m_pKey) {
-		m_pKey->update(this);
-		g_hashTable.insert(*m_pKey, m_pData);
+		updateHashKey();
+		insertHashKey();
 	}
 
 	return true;
@@ -471,7 +513,7 @@ void qtractorMidiClip::closeMidiFile (void)
 	if (m_pData) {
 		m_pData->detach(this);
 		if (m_pData->count() < 1) {
-			if (m_pKey) g_hashTable.remove(*m_pKey);
+			removeHashKey();
 			delete m_pData;
 		}
 		m_pData = NULL;
@@ -494,10 +536,19 @@ QString qtractorMidiClip::createFilePathRevision ( bool bForce )
 {
 	QString sFilename = filename();
 
+	// Check file-hash reference...
+	if (m_iRevision > 0 && m_pKey) {
+		FileKey fkey(m_pKey);
+		FileHash::ConstIterator fiter = g_hashFiles.constFind(fkey);
+		if (fiter != g_hashFiles.constEnd() && fiter.value() > 1)
+			m_iRevision = 0;
+	}
+
 	if (m_iRevision == 0 || bForce) {
+		qtractorTrack *pTrack = track();
 		qtractorSession *pSession = qtractorSession::getInstance();
-		if (pSession && track())
-			sFilename = pSession->createFilePath(track()->trackName(), "mid");
+		if (pTrack && pSession)
+			sFilename = pSession->createFilePath(pTrack->trackName(), "mid");
 		sFilename = qtractorMidiFile::createFilePathRevision(sFilename);
 	#ifdef CONFIG_DEBUG
 		qDebug("qtractorMidiClip::createFilePathRevision(\"%s\")",
@@ -517,7 +568,7 @@ void qtractorMidiClip::setFilenameEx ( const QString& sFilename )
 	if (m_pData == NULL)
 		return;
 
-	if (m_pKey) g_hashTable.remove(*m_pKey);
+	removeHashKey();
 
 	QListIterator<qtractorMidiClip *> iter(m_pData->clips());
 	while (iter.hasNext()) {
@@ -528,7 +579,7 @@ void qtractorMidiClip::setFilenameEx ( const QString& sFilename )
 		pMidiClip->updateEditor(true);
 	}
 
-	if (m_pKey) g_hashTable.insert(*m_pKey, m_pData);
+	insertHashKey();
 }
 
 
@@ -538,7 +589,7 @@ void qtractorMidiClip::setClipLengthEx ( unsigned long iClipLength )
 	if (m_pData == NULL)
 		return;
 
-	if (m_pKey) g_hashTable.remove(*m_pKey);
+	removeHashKey();
 
 	QListIterator<qtractorMidiClip *> iter(m_pData->clips());
 	while (iter.hasNext()) {
@@ -547,7 +598,7 @@ void qtractorMidiClip::setClipLengthEx ( unsigned long iClipLength )
 		pMidiClip->updateHashKey();
 	}
 
-	if (m_pKey) g_hashTable.insert(*m_pKey, m_pData);
+	insertHashKey();
 }
 
 
@@ -585,13 +636,44 @@ void qtractorMidiClip::setDirtyEx ( bool bDirty )
 }
 
 
-// Update local hash key.
+// Manage local hash key.
+void qtractorMidiClip::insertHashKey (void)
+{
+	if (m_pKey) {
+		// Increment file-hash reference...
+		FileKey fkey(m_pKey);
+		FileHash::Iterator fiter = g_hashFiles.find(fkey);
+		if (fiter == g_hashFiles.end())
+			fiter =  g_hashFiles.insert(fkey, 0);
+		++fiter.value();
+		// Insert actual clip-hash reference....
+		g_hashTable.insert(*m_pKey, m_pData);
+	}
+}
+
+
 void qtractorMidiClip::updateHashKey (void)
 {
 	if (m_pKey) m_pKey->update(this);
 }
 
 
+void qtractorMidiClip::removeHashKey (void)
+{
+	if (m_pKey) {
+		// Decrement file-hash reference...
+		FileKey fkey(m_pKey);
+		FileHash::Iterator fiter = g_hashFiles.find(fkey);
+		if (fiter != g_hashFiles.end()) {
+			if (--fiter.value() < 1)
+				g_hashFiles.remove(fkey);
+		}
+		// Remove actual clip-hash reference....
+		g_hashTable.remove(*m_pKey);
+	}
+}
+
+
 // Update (clone) local hash data.
 void qtractorMidiClip::updateHashData (void)
 {
@@ -622,10 +704,15 @@ void qtractorMidiClip::updateHashData (void)
 	m_pData = pNewData;
 	m_pData->attach(this);
 
-	if (m_pKey) {
-		m_pKey->update(this);
-		g_hashTable.insert(*m_pKey, m_pData);
-	}
+	updateHashKey();
+	insertHashKey();
+}
+
+
+// Whether local hash is being shared.
+bool qtractorMidiClip::isHashLinked (void) const
+{
+	return (m_pData && m_pData->count() > 1);
 }
 
 
@@ -633,6 +720,7 @@ void qtractorMidiClip::updateHashData (void)
 void qtractorMidiClip::clearHashTable (void)
 {
 	g_hashTable.clear();
+	g_hashFiles.clear();
 }
 
 
@@ -664,28 +752,20 @@ void qtractorMidiClip::seek ( unsigned long iFrame )
 
 
 // Reset clip state.
-void qtractorMidiClip::reset ( bool bLooping )
+void qtractorMidiClip::reset ( bool /* bLooping */ )
 {
-#ifdef CONFIG_DEBUG_0
-	qDebug("qtractorMidiClip[%p]::reset(%d)", this, (int) bLooping);
-#endif
-
 	qtractorMidiSequence *pSeq = sequence();
 	if (pSeq == NULL)
 		return;
 
 	// Reset to the first sequence event...
 	m_playCursor.reset(pSeq);
-
-	// Take the time from loop-start?
-	if (bLooping && clipLoopStart() < clipLoopEnd())
-		seek(clipLoopStart());
 }
 
 
 // Loop positioning.
-void qtractorMidiClip::set_loop ( unsigned long /* iLoopStart */,
-	unsigned long /* iLoopEnd */ )
+void qtractorMidiClip::setLoop (
+	unsigned long /* iLoopStart */, unsigned long /* iLoopEnd */ )
 {
 	// Do nothing?
 }
@@ -819,8 +899,8 @@ void qtractorMidiClip::draw ( QPainter *pPainter, const QRect& clipRect,
 
 	// Check maximum note span...
 	int iNoteSpan = (pSeq->noteMax() - pSeq->noteMin()) + 1;
-	if (iNoteSpan < 4)
-		iNoteSpan = 4;
+	if (iNoteSpan < 6)
+		iNoteSpan = 6;
 
 	unsigned long iFrameStart = clipStart() + iClipOffset;
 	int cx = pSession->pixelFromFrame(iFrameStart);
@@ -834,11 +914,11 @@ void qtractorMidiClip::draw ( QPainter *pPainter, const QRect& clipRect,
 	pNode = cursor.seekPixel(cx + clipRect.width());	
 	unsigned long iTimeEnd = pNode->tickFromPixel(cx + clipRect.width());
 
-	const QColor& fg = track()->foreground();
+	const QColor& fg = pTrack->foreground();
 	pPainter->setPen(fg);
 	pPainter->setBrush(fg.lighter());
 
-	bool bClipRecord = (track()->clipRecord() == this);
+	bool bClipRecord = (pTrack->clipRecord() == this);
 	int h1 = clipRect.height() - 3;
 	int h  = h1 / iNoteSpan;
 	if (h < 3) h = 3;
@@ -955,32 +1035,13 @@ bool qtractorMidiClip::queryEditor (void)
 	if (m_pMidiEditorForm)
 		return m_pMidiEditorForm->queryClose();
 
-	qtractorSession *pSession = qtractorSession::getInstance();
-	if (pSession == NULL)
-		return false;
-
-	bool bQueryEditor = qtractorClip::queryEditor();
-
 	// Are any dirty changes pending commit?
+	bool bQueryEditor = qtractorClip::queryEditor();
 	if (!bQueryEditor) {
 		switch (qtractorMidiEditorForm::querySave(filename())) {
 		case QMessageBox::Save:	{
-			// Have a new filename revision...
-			const QString& sFilename = createFilePathRevision();
 			// Save/replace the clip track...
-			bQueryEditor = qtractorMidiFile::saveCopyFile(
-				sFilename, filename(), trackChannel(), format(),
-				sequence(),	pSession->timeScale(),
-				pSession->tickFromFrame(clipStart()));
-			if (bQueryEditor) {
-				// Pre-commit dirty changes...
-				setFilenameEx(sFilename);
-				// Reference for immediate file addition...
-				qtractorMainForm *pMainForm
-					= qtractorMainForm::getInstance();
-				if (pMainForm)
-					pMainForm->addMidiFile(sFilename);
-			}
+			bQueryEditor = saveCopyFile();
 			break;
 		}
 		case QMessageBox::Discard:
@@ -1021,6 +1082,34 @@ QString qtractorMidiClip::toolTip (void) const
 }
 
 
+// Auto-save to (possible) new file revision.
+bool qtractorMidiClip::saveCopyFile (void)
+{
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	// Have a new filename revision...
+	const QString& sFilename = createFilePathRevision();
+
+	// Save/replace the clip track...
+	if (!qtractorMidiFile::saveCopyFile(
+			sFilename, filename(), trackChannel(), format(), sequence(),
+			pSession->timeScale(), pSession->tickFromFrame(clipStart())))
+		return false;
+
+	// Pre-commit dirty changes...
+	setFilenameEx(sFilename);
+
+	// Reference for immediate file addition...
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm)
+		pMainForm->addMidiFile(sFilename);
+
+	return true;
+}
+
+
 // Virtual document element methods.
 bool qtractorMidiClip::loadClipElement (
 	qtractorDocument * /* pDocument */, QDomElement *pElement )
