@@ -106,7 +106,7 @@ public:
 	void reset(bool bLooping);
 
 	// Loop positioning.
-	void set_loop(unsigned long iLoopStart, unsigned long iLoopEnd);
+	void setLoop(unsigned long iLoopStart, unsigned long iLoopEnd);
 
 	// Clip close-commit (record specific)
 	void close();
@@ -127,6 +127,9 @@ public:
 	// MIDI clip tool-tip.
 	QString toolTip() const;
 
+	// Auto-save to (possible) new file revision.
+	bool saveCopyFile();
+
 	// MIDI clip export method.
 	typedef void (*ClipExport)(qtractorMidiSequence *, void *);
 
@@ -138,6 +141,11 @@ public:
 	static void setDefaultFormat(unsigned short iFormat);
 	static unsigned short defaultFormat();
 
+	// MIDI file hash key.
+	class FileKey;
+
+	typedef QHash<FileKey, int> FileHash;
+
 	// Most interesting key/data (ref-counted?)...
 	class Key;
 	class Data
@@ -207,12 +215,17 @@ public:
 	// Sync all ref-counted clip-dirtyness.
 	void setDirtyEx(bool bDirty);
 
-	// Update local hash key.
+	// Manage local hash key.
+	void insertHashKey();
 	void updateHashKey();
+	void removeHashKey();
 
 	// Update (clone) local hash data.
 	void updateHashData();
 
+	// Whether local hash is being shared.
+	bool isHashLinked() const;
+
 	// Make sure the clip hash-table gets reset.
 	static void clearHashTable();
 
@@ -247,6 +260,9 @@ private:
 
 	static Hash g_hashTable;
 
+	// MIDI file hash key.
+	static FileHash g_hashFiles;
+
 	// To optimize and keep track of current playback
 	// position, mostly like an sequence cursor/iterator.
 	qtractorMidiCursor m_playCursor;
