@@ -1,7 +1,7 @@
 // qtractorMidiCursor.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -46,7 +46,8 @@ qtractorMidiEvent *qtractorMidiCursor::seek (
 		// Seek forward...
 		if (m_pEvent == NULL)
 			m_pEvent = pSeq->events().first();
-		while (m_pEvent && m_pEvent->time() < iTime)
+		while (m_pEvent && m_pEvent->next()
+			&& (m_pEvent->next())->time() < iTime)
 			m_pEvent = m_pEvent->next();
 		if (m_pEvent == NULL)
 			m_pEvent = pSeq->events().last();
