@@ -1,7 +1,7 @@
 // qtractorMidiSequence.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -40,7 +40,7 @@ class qtractorMidiSequence
 public:
 
 	// Constructor.
-	qtractorMidiSequence(const QString& sName = QString::null,
+	qtractorMidiSequence(const QString& sName = QString(),
 		unsigned short iChannel = 0, unsigned short iTicksPerBeat = 960);
 
 	// Destructor.
@@ -112,6 +112,9 @@ public:
 	void replaceEvents(qtractorMidiSequence *pSeq,
 		unsigned long iTimeOffset = 0, unsigned long iTimeLength = 0);
 
+	// Clopy all events from another sequence (raw-copy).
+	void copyEvents(qtractorMidiSequence *pSeq);
+
 	// Sequence closure method.
 	void close();
 
@@ -133,7 +136,7 @@ private:
 	int            m_iBank;
 	int            m_iProgram;
 
-	// Statictical helper variables.
+	// Statistical helper variables.
 	unsigned char  m_noteMin;
 	unsigned char  m_noteMax;
 	unsigned long  m_duration;
