@@ -1946,6 +1946,7 @@ void qtractorPluginParam::updateValue ( float fValue, bool bUpdate )
 qtractorPluginParam::Observer::Observer ( qtractorPluginParam *pParam )
 	: qtractorMidiControlObserver(pParam->subject()), m_pParam(pParam)
 {
+	setCurveList((pParam->plugin())->list()->curveList());
 }
 
 
@@ -1957,6 +1958,8 @@ void qtractorPluginParam::Observer::update (void)
 	qtractorPlugin *pPlugin = m_pParam->plugin();
 	if (pPlugin->directAccessParamIndex() == long(m_pParam->index()))
 		pPlugin->updateDirectAccessParam();
+
+	pPlugin->updateParam(m_pParam, qtractorMidiControlObserver::value());
 }
 
 
