@@ -95,8 +95,10 @@ public:
 	void paint(QPainter *pPainter, const QStyleOptionViewItem& option,
 		const QModelIndex& index) const
 	{
-		// Unselectable items get special grayed out painting...
-		QListWidgetItem *pItem = m_pListWidget->item(index.row());
+		qtractorPluginListItem *pItem
+			= static_cast<qtractorPluginListItem *> (
+				m_pListWidget->item(index.row()));
+		// nb. Unselectable items get special grayed out painting...
 		if (pItem) {
 			pPainter->save();
 			const QPalette& pal = option.palette;
@@ -111,9 +113,32 @@ public:
 			}
 			// Fill the background...
 			pPainter->fillRect(option.rect, rgbBack);
+			const QSize& iconSize = m_pListWidget->iconSize();
+			// Draw the direct access parameter value status...
+			QPolygon polyg(3);
+			qtractorPlugin *pPlugin = pItem->plugin();
+			qtractorPluginParam *pDirectAccessParam = NULL;
+			qtractorMidiControlObserver *pDirectAccessObserver = NULL;
+			if (pPlugin)
+				pDirectAccessParam = pPlugin->directAccessParam();
+			if (pDirectAccessParam)
+				pDirectAccessObserver = pDirectAccessParam->observer();
+			if (pDirectAccessObserver) {
+				float fScale = pDirectAccessObserver->scaleFromValue(
+					pDirectAccessParam->value(),
+					pDirectAccessParam->isLogarithmic());
+				QRect rectValue = option.rect
+					.adjusted(iconSize.width(), 1, -2, -2);
+				const int iDirectAccessWidth = rectValue.width();
+				pItem->setDirectAccessWidth(iDirectAccessWidth);
+				rectValue.setWidth(int(fScale * float(iDirectAccessWidth)));
+				pPainter->fillRect(rectValue, rgbBack.lighter(140));
+				polyg.setPoint(0, rectValue.right(), rectValue.top() + 1);
+				polyg.setPoint(1, rectValue.right() - 2, rectValue.bottom() + 1);
+				polyg.setPoint(2, rectValue.right() + 2, rectValue.bottom() + 1);
+			}
 			// Draw the icon...
 			QRect rect = option.rect;
-			const QSize& iconSize = m_pListWidget->iconSize();
 			pPainter->drawPixmap(1,
 				rect.top() + ((rect.height() - iconSize.height()) >> 1),
 				pItem->icon().pixmap(iconSize));
@@ -131,6 +156,14 @@ public:
 			pPainter->drawLine(
 				option.rect.left(),  option.rect.bottom(),
 				option.rect.right(), option.rect.bottom());
+			if (pDirectAccessObserver) {
+				pPainter->setRenderHint(QPainter::Antialiasing, true);
+				pPainter->setBrush(rgbBack.darker(160));
+				pPainter->drawPolygon(polyg);
+				pPainter->setPen(rgbBack.lighter(180));
+				pPainter->drawLine(polyg.at(0), polyg.at(1));
+				pPainter->setRenderHint(QPainter::Antialiasing, false);
+			}
 			pPainter->restore();
 		//	if (option.state & QStyle::State_HasFocus)
 		//		QItemDelegate::drawFocus(pPainter, option, option.rect);
@@ -151,17 +184,25 @@ private:
 
 // Constructors.
 qtractorPluginListItem::qtractorPluginListItem ( qtractorPlugin *pPlugin )
-	: QListWidgetItem()
+	: QListWidgetItem(), m_pPlugin(pPlugin), m_iDirectAccessWidth(0)
 {
-	m_pPlugin = pPlugin;
-
 	m_pPlugin->addItem(this);
 
-	QListWidgetItem::setText((m_pPlugin->type())->name());
+	QString sText;
+	qtractorPluginType *pType = m_pPlugin->type();
+	if (pType->typeHint() == qtractorPluginType::AuxSend) {
+		qtractorAuxSendPlugin *pAuxSendPlugin
+			= static_cast<qtractorAuxSendPlugin *> (m_pPlugin);
+		if (pAuxSendPlugin)
+			sText = pAuxSendPlugin->audioBusName();
+	}
+
+	QListWidgetItem::setText(sText.isEmpty() ? pType->name() : sText);
 
 	updateActivated();
 }
 
+
 // Destructor.
 qtractorPluginListItem::~qtractorPluginListItem (void)
 {
@@ -201,6 +242,8 @@ qtractorPluginListView::qtractorPluginListView ( QWidget *pParent )
 	}
 
 	// Drag-and-drop stuff.
+	m_dragCursor  = DragNone;
+	m_dragState   = DragNone;
 	m_pDragItem   = NULL;
 	m_pDropItem   = NULL;
 	m_pRubberBand = NULL;
@@ -213,6 +256,7 @@ qtractorPluginListView::qtractorPluginListView ( QWidget *pParent )
 	QListWidget::setIconSize(QSize(10, 10));
 	QListWidget::setItemDelegate(new qtractorPluginListItemDelegate(this));
 	QListWidget::setSelectionMode(QAbstractItemView::SingleSelection);
+	QListWidget::setMouseTracking(true);
 
 	QListWidget::viewport()->setBackgroundRole(QPalette::Window);
 
@@ -259,10 +303,8 @@ void qtractorPluginListView::setPluginList ( qtractorPluginList *pPluginList )
 
 	m_pPluginList = pPluginList;
 
-	if (m_pPluginList) {
-		m_pPluginList->setAutoDelete(true);
+	if (m_pPluginList)
 		m_pPluginList->addView(this);
-	}
 
 	refresh();
 }
@@ -407,8 +449,7 @@ void qtractorPluginListView::addPlugin (void)
 		return;
 
 	qtractorPluginSelectForm selectForm(this);
-	selectForm.setChannels(m_pPluginList->channels(),
-		(m_pPluginList->flags() & qtractorPluginList::Midi));
+	selectForm.setChannels(m_pPluginList->channels(), m_pPluginList->isMidi());
 	if (!selectForm.exec())
 		return;
 
@@ -434,7 +475,10 @@ void qtractorPluginListView::addPlugin (void)
 			pPlugin->setActivated(selectForm.isPluginActivated());
 			pAddPluginCommand->addPlugin(pPlugin);
 			// Show the plugin form right away...
-			(pPlugin->form())->activateForm();
+			if ((pPlugin->type())->isEditor())
+				pPlugin->openEditor(this);
+			else
+				(pPlugin->form())->activateForm();
 		}
 	}
 
@@ -466,6 +510,7 @@ void qtractorPluginListView::addInsertPlugin (void)
 			QString(), // Empty filename!
 			m_pPluginList->channels(),
 			qtractorPluginType::Insert);
+
 	if (pPlugin) {
 		// Show the plugin form right away...
 		(pPlugin->form())->activateForm();
@@ -480,6 +525,40 @@ void qtractorPluginListView::addInsertPlugin (void)
 }
 
 
+// Add an insert pseudo-plugin slot.
+void qtractorPluginListView::addAuxSendPlugin (void)
+{
+	if (m_pPluginList == NULL)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	// Tell the world we'll take some time...
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	// Create our special pseudo-plugin type...
+	qtractorPlugin *pPlugin
+		= qtractorPluginFile::createPlugin(m_pPluginList,
+			QString(), // Empty filename!
+			m_pPluginList->channels(),
+			qtractorPluginType::AuxSend);
+
+	if (pPlugin) {
+		// Show the plugin form right away...
+		(pPlugin->form())->activateForm();
+		// Make it a undoable command...
+		pSession->execute(new qtractorAddAuxSendPluginCommand(pPlugin));
+	}
+
+	// We're formerly done.
+	QApplication::restoreOverrideCursor();
+
+	emit contentsChanged();
+}
+
+
 // Remove an existing plugin slot.
 void qtractorPluginListView::removePlugin (void)
 {
@@ -670,6 +749,38 @@ void qtractorPluginListView::moveDownPlugin (void)
 }
 
 
+// Select a direct access parameter index.
+void qtractorPluginListView::directAccessPlugin (void)
+{
+	if (m_pPluginList == NULL)
+		return;
+
+	qtractorPluginListItem *pItem
+		= static_cast<qtractorPluginListItem *> (QListWidget::currentItem());
+	if (pItem == NULL)
+		return;
+
+	qtractorPlugin *pPlugin = pItem->plugin();
+	if (pPlugin == NULL)
+		return;
+
+	// Retrieve direct access parameter index from action data...
+	QAction *pAction = qobject_cast<QAction *> (sender());
+	if (pAction == NULL)
+		return;
+
+	int iDirectAccessParamIndex = pAction->data().toInt();
+
+	// Make it a undoable command...
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+	
+	pSession->execute(
+		new qtractorDirectAccessParamCommand(pPlugin, iDirectAccessParamIndex));
+}
+
+
 // Show/hide an existing plugin form slot.
 void qtractorPluginListView::propertiesPlugin (void)
 {
@@ -710,7 +821,7 @@ void qtractorPluginListView::editPlugin (void)
 	if (pPlugin->isEditorVisible())
 		pPlugin->closeEditor();
 	else
-		pPlugin->openEditor(pPlugin->form());
+		pPlugin->openEditor(this);
 }
 
 
@@ -787,7 +898,28 @@ void qtractorPluginListView::audioOutputBus (void)
 
 	pSession->execute(
 		new qtractorAudioOutputBusCommand(pMidiManager,
-			!pMidiManager->isAudioOutputBus()));
+			!pMidiManager->isAudioOutputBus(), // Toggle!
+			pMidiManager->isAudioOutputAutoConnect()));
+
+	emit contentsChanged();
+}
+
+
+void qtractorPluginListView::audioOutputAutoConnect (void)
+{
+	qtractorMidiManager *pMidiManager = m_pPluginList->midiManager();
+	if (pMidiManager == NULL)
+		return;
+
+	// Make it an undoable command...
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pSession->execute(
+		new qtractorAudioOutputBusCommand(pMidiManager,
+			pMidiManager->isAudioOutputBus(),
+			!pMidiManager->isAudioOutputAutoConnect())); // Toggle!
 
 	emit contentsChanged();
 }
@@ -813,7 +945,17 @@ void qtractorPluginListView::itemActivatedSlot ( QListWidgetItem *item )
 	if (pPlugin == NULL)
 		return;
 
-	(pPlugin->form())->activateForm();
+	if ((pPlugin->type())->isEditor()) {
+		if (pPlugin->isEditorVisible())
+			pPlugin->closeEditor();
+		else
+			pPlugin->openEditor(this);
+	} else {
+		if (pPlugin->isFormVisible())
+			(pPlugin->form())->hide();
+		else
+			(pPlugin->form())->activateForm();
+	}
 }
 
 
@@ -821,22 +963,39 @@ void qtractorPluginListView::itemActivatedSlot ( QListWidgetItem *item )
 bool qtractorPluginListView::eventFilter ( QObject *pObject, QEvent *pEvent )
 {
 	QWidget *pViewport = QListWidget::viewport();
-	if (static_cast<QWidget *> (pObject) == pViewport
-		&& pEvent->type() == QEvent::ToolTip) {
-		QHelpEvent *pHelpEvent = static_cast<QHelpEvent *> (pEvent);
-		if (pHelpEvent) {
-			qtractorPluginListItem *pItem
-				= static_cast<qtractorPluginListItem *> (
-					QListWidget::itemAt(pHelpEvent->pos()));
-			qtractorPlugin *pPlugin = NULL;
-			if (pItem)
-				pPlugin = pItem->plugin();
-			if (pPlugin) {
-				QToolTip::showText(pHelpEvent->globalPos(),
-					(pPlugin->type())->name(), pViewport);
-				return true;
+	if (static_cast<QWidget *> (pObject) == pViewport) {
+		if (pEvent->type() == QEvent::ToolTip) {
+			QHelpEvent *pHelpEvent = static_cast<QHelpEvent *> (pEvent);
+			if (pHelpEvent) {
+				qtractorPluginListItem *pItem
+					= static_cast<qtractorPluginListItem *> (
+						QListWidget::itemAt(pHelpEvent->pos()));
+				qtractorPlugin *pPlugin = NULL;
+				if (pItem)
+					pPlugin = pItem->plugin();
+				if (pPlugin) {
+					QString sToolTip = pItem->text(); // (pPlugin->type())->name();
+					if (pPlugin->isDirectAccessParam()) {
+						qtractorPluginParam *pDirectAccessParam
+							= pPlugin->directAccessParam();
+						if (pDirectAccessParam) {
+							sToolTip.append(QString("\n(%1: %2)")
+								.arg(pDirectAccessParam->name())
+								.arg(pDirectAccessParam->value(), 0, 'g', 3));
+						}
+					}
+					QToolTip::showText(pHelpEvent->globalPos(),
+						sToolTip, pViewport);
+					return true;
+				}
 			}
 		}
+		else
+		if (pEvent->type() == QEvent::Leave) {
+			m_dragCursor = DragNone;
+			QListWidget::unsetCursor();
+			return true;
+		}
 	}
 
 	// Not handled here.
@@ -844,6 +1003,37 @@ bool qtractorPluginListView::eventFilter ( QObject *pObject, QEvent *pEvent )
 }
 
 
+// trap the wheel event to change the value of the direcgAccessParameter
+void qtractorPluginListView::wheelEvent ( QWheelEvent *pWheelEvent )
+{
+	const QPoint& pos = pWheelEvent->pos();
+	qtractorPluginListItem *pItem
+		= static_cast<qtractorPluginListItem *> (QListWidget::itemAt(pos));
+	if (pItem) {
+		qtractorPlugin *pPlugin = pItem->plugin();
+		qtractorPluginParam *pDirectAccessParam = NULL;
+		qtractorMidiControlObserver *pDirectAccessObserver = NULL;
+		if (pPlugin)
+			pDirectAccessParam = pPlugin->directAccessParam();
+		if (pDirectAccessParam)
+			pDirectAccessObserver = pDirectAccessParam->observer();
+		if (pDirectAccessObserver) {
+			const bool bLogarithmic = pDirectAccessParam->isLogarithmic();
+			float fValue = pDirectAccessObserver->value();
+			float fScale = pDirectAccessObserver->scaleFromValue(
+				fValue, bLogarithmic);
+			float fStep = (pWheelEvent->delta() > 0 ? 1.0f : -1.0f);
+			if (!pDirectAccessParam->isInteger())
+				fStep *= 0.1f;
+			fScale += fStep;
+			fValue = pDirectAccessObserver->valueFromScale(
+				fScale, bLogarithmic);
+			pDirectAccessParam->updateValue(fValue, true);
+		}
+	}
+}
+
+
 // To get precize clicking for in-place (de)activation;
 // handle mouse events for drag-and-drop stuff.
 void qtractorPluginListView::mousePressEvent ( QMouseEvent *pMouseEvent )
@@ -860,6 +1050,7 @@ void qtractorPluginListView::mousePressEvent ( QMouseEvent *pMouseEvent )
 	if (pMouseEvent->button() == Qt::LeftButton) {
 		m_posDrag   = pos;
 		m_pDragItem = pItem;
+		dragDirectAccess(pos);
 	}
 
 	QListWidget::mousePressEvent(pMouseEvent);	
@@ -870,20 +1061,31 @@ void qtractorPluginListView::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 {
 	QListWidget::mouseMoveEvent(pMouseEvent);
 
-	if ((pMouseEvent->buttons() & Qt::LeftButton) && m_pDragItem
-		&& ((pMouseEvent->pos() - m_posDrag).manhattanLength()
-			>= QApplication::startDragDistance())) {
-		// We'll start dragging something alright...
-		QMimeData *pMimeData = new QMimeData();
-		encodeItem(pMimeData, m_pDragItem);
-		QDrag *pDrag = new QDrag(this);
-		pDrag->setMimeData(pMimeData);
-		pDrag->setPixmap(m_pDragItem->icon().pixmap(16));
-		pDrag->setHotSpot(QPoint(-4, -12));
-		pDrag->start(Qt::MoveAction);
-		// We've dragged and maybe dropped it by now...
-		m_pDragItem = NULL;
+	const QPoint& pos = pMouseEvent->pos();
+	if ((pMouseEvent->buttons() & Qt::LeftButton) && m_pDragItem) {
+		if (m_dragCursor != DragNone)
+			m_dragState = m_dragCursor;
+		if (m_dragState != DragNone) {
+			dragDirectAccess(pos);
+			return;
+		}
+		if ((pos - m_posDrag).manhattanLength()
+			>= QApplication::startDragDistance()) {
+			// We'll start dragging something alright...
+			QMimeData *pMimeData = new QMimeData();
+			encodeItem(pMimeData, m_pDragItem);
+			QDrag *pDrag = new QDrag(this);
+			pDrag->setMimeData(pMimeData);
+			pDrag->setPixmap(m_pDragItem->icon().pixmap(16));
+			pDrag->setHotSpot(QPoint(-4, -12));
+			pDrag->start(Qt::MoveAction);
+			// We've dragged and maybe dropped it by now...
+			m_pDragItem = NULL;
+		}
 	}
+	else
+	if (m_dragState == DragNone)
+		dragDirectAccess(pos);
 }
 
 
@@ -894,7 +1096,6 @@ void qtractorPluginListView::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 	const QPoint& pos = pMouseEvent->pos();
 	qtractorPluginListItem *pItem
 		= static_cast<qtractorPluginListItem *> (QListWidget::itemAt(pos));
-
 	if (pItem && pos.x() > 0 && pos.x() < QListWidget::iconSize().width()
 		&& m_pClickedItem == pItem) {
 		qtractorPlugin *pPlugin = pItem->plugin();
@@ -947,8 +1148,8 @@ bool qtractorPluginListView::canDropEvent ( QDropEvent *pDropEvent )
 		return false;
 
 	// All that to check whether it will get properly instantiated.
-	// FIXME: We don't care whether it's a MIDI chain at this time...
-	if ((pPlugin->type())->instances(m_pPluginList->channels(), false) < 1)
+	if ((pPlugin->type())->instances(
+			m_pPluginList->channels(), m_pPluginList->isMidi()) < 1)
 		return false;
 
 	// This is the place we'll drop something
@@ -1027,10 +1228,18 @@ void qtractorPluginListView::dragLeaveEvent ( QDragLeaveEvent * )
 		m_pRubberBand = NULL;
 	}
 
+	// Should fallback mouse cursor...
+	if (m_dragCursor != DragNone)
+		QListWidget::unsetCursor();
+
+	m_dragCursor = DragNone;
+	m_dragState  = DragNone;
+
 	m_pClickedItem = NULL;
 
 	m_pDragItem = NULL;
 	m_pDropItem = NULL;
+
 }
 
 
@@ -1155,6 +1364,10 @@ void qtractorPluginListView::contextMenuEvent (
 		QIcon(":/images/formAdd.png"),
 		tr("Add &Insert"), this, SLOT(addInsertPlugin()));
 	pAction->setEnabled(m_pPluginList->channels() > 0);
+	pAction = pInsertMenu->addAction(
+		QIcon(":/images/formAdd.png"),
+		tr("Add &Aux Send"), this, SLOT(addAuxSendPlugin()));
+	pAction->setEnabled(m_pPluginList->channels() > 0);
 	pInsertMenu->addSeparator();
 	bool bInsertEnabled = (pPlugin
 		&& (pPlugin->type())->typeHint() == qtractorPluginType::Insert);
@@ -1170,7 +1383,7 @@ void qtractorPluginListView::contextMenuEvent (
 	menu.addSeparator();
 
 	pAction = menu.addAction(
-		tr("Act&ivate"), this, SLOT(activatePlugin()));
+		tr("Ac&tivate"), this, SLOT(activatePlugin()));
 	pAction->setCheckable(true);
 	pAction->setChecked(pPlugin && pPlugin->isActivated());
 	pAction->setEnabled(pPlugin != NULL);
@@ -1214,6 +1427,35 @@ void qtractorPluginListView::contextMenuEvent (
 
 	menu.addSeparator();
 
+	QMenu *pDirectAccessParamMenu = menu.addMenu("Dire&ct Access");
+	if (pPlugin) {
+		int iDirectAccessParamIndex = pPlugin->directAccessParamIndex();
+		const qtractorPlugin::Params& params = pPlugin->params();
+		qtractorPlugin::Params::ConstIterator param = params.constBegin();
+		for ( ; param != params.constEnd(); ++param) {
+			qtractorPluginParam *pParam = param.value();
+			int iParamIndex = int(param.key());
+			pAction = pDirectAccessParamMenu->addAction(
+				pParam->name(), this, SLOT(directAccessPlugin()));
+			pAction->setCheckable(true);
+			pAction->setChecked(iDirectAccessParamIndex == iParamIndex);
+			pAction->setData(iParamIndex);
+		}
+		bool bParams = (params.count() > 0);
+		if (bParams) {
+			pDirectAccessParamMenu->addSeparator();
+			pAction = pDirectAccessParamMenu->addAction(
+				tr("&None"), this, SLOT(directAccessPlugin()));
+			pAction->setCheckable(true);
+			pAction->setChecked(iDirectAccessParamIndex < 0);
+			pAction->setData(int(-1));
+		}
+		pDirectAccessParamMenu->setEnabled(bParams);
+	}
+	else pDirectAccessParamMenu->setEnabled(false);
+
+	menu.addSeparator();
+
 	pAction = menu.addAction(
 		QIcon(":/images/trackProperties.png"),
 		tr("&Properties..."), this, SLOT(propertiesPlugin()));
@@ -1240,6 +1482,11 @@ void qtractorPluginListView::contextMenuEvent (
 			tr("&Dedicated"), this, SLOT(audioOutputBus()));
 		pAction->setCheckable(true);
 		pAction->setChecked(pMidiManager->isAudioOutputBus());
+		pAction = pAudioMenu->addAction(
+			tr("&Auto-connect"), this, SLOT(audioOutputAutoConnect()));
+		pAction->setCheckable(true);
+		pAction->setChecked(pMidiManager->isAudioOutputAutoConnect());
+		pAction->setEnabled(pMidiManager->isAudioOutputBus());
 	}
 
 	menu.exec(pContextMenuEvent->globalPos());
@@ -1284,4 +1531,53 @@ qtractorPluginListItem *qtractorPluginListView::decodeItem (
 }
 
 
+// Direct access parameter handle.
+void qtractorPluginListView::dragDirectAccess ( const QPoint& pos )
+{
+	qtractorPluginListItem *pItem = m_pDragItem;
+	if (pItem == NULL)
+		pItem = static_cast<qtractorPluginListItem *> (QListWidget::itemAt(pos));
+	if (pItem == NULL)
+		return;
+
+	qtractorPlugin *pPlugin = pItem->plugin();
+	if (pPlugin == NULL)
+		return;
+
+	qtractorPluginParam *pDirectAccessParam	
+		= pPlugin->directAccessParam();
+	if (pDirectAccessParam == NULL)
+		return;
+
+	qtractorMidiControlObserver *pDirectAccessObserver
+		= pDirectAccessParam->observer();
+	if (pDirectAccessObserver == NULL)
+		return;
+
+	const bool bLogarithmic = pDirectAccessParam->isLogarithmic();
+	const int iDirectAccessWidth = pItem->directAccessWidth();
+	const QRect& rectItem = QListWidget::visualItemRect(pItem)
+		.adjusted(QListWidget::iconSize().width(), 1, -2, -2);
+
+	if (m_dragState == DragNone) {
+		float fValue = pDirectAccessParam->value();
+		float fScale = pDirectAccessObserver->scaleFromValue(fValue, bLogarithmic);
+		int x = rectItem.x() + int(fScale * float(iDirectAccessWidth));
+		if (pos.x() > x - 4 && pos.x() < x + 4) {
+			m_dragCursor = DragDirectAccess;
+			QListWidget::setCursor(QCursor(Qt::PointingHandCursor));
+		} else {
+			QListWidget::unsetCursor();
+			m_dragCursor = DragNone;
+		}
+	}
+	else
+	if (m_dragState == DragDirectAccess) {
+		float fScale = float(pos.x() - rectItem.x()) / float(iDirectAccessWidth);
+		float fValue = pDirectAccessObserver->valueFromScale(fScale, bLogarithmic);
+		pDirectAccessParam->updateValue(fValue, true);
+	}
+}
+
+
 // end of qtractorPluginListView.cpp
