@@ -132,6 +132,8 @@ qtractorSession::qtractorSession (void)
 
 	m_bAutoTimeStretch  = false;
 
+	m_iLoopRecordingMode = 0;
+
 	clear();
 }
 
@@ -1417,10 +1419,11 @@ void qtractorSession::setRecording ( bool bRecording )
 			iClipStart = iPunchIn;
 	}
 
+	unsigned long iFrameTime = frameTimeEx();
 	for (qtractorTrack *pTrack = m_tracks.first();
 			pTrack; pTrack = pTrack->next()) {
 		if (pTrack->isRecord())
-			trackRecord(pTrack, bRecording, iClipStart);
+			trackRecord(pTrack, bRecording, iClipStart, iFrameTime);
 	}
 }
 
@@ -1430,13 +1433,33 @@ bool qtractorSession::isRecording (void) const
 }
 
 
+// Loop-recording/take mode.
+void qtractorSession::setLoopRecordingMode ( int iLoopRecordingMode )
+{
+	m_iLoopRecordingMode = iLoopRecordingMode;
+}
+
+int qtractorSession::loopRecordingMode (void) const
+{
+	return m_iLoopRecordingMode;
+}
+
+// Loop-recording/take state.
+bool qtractorSession::isLoopRecording (void) const
+{
+	return isLooping() && (m_iLoopRecordingMode > 0);
+}
+
+
 // Immediate track record-arming.
 void qtractorSession::trackRecord (
-	qtractorTrack *pTrack, bool bRecord, unsigned long iClipStart )
+	qtractorTrack *pTrack, bool bRecord,
+	unsigned long iClipStart, unsigned long iFrameTime )
 {
 #ifdef CONFIG_DEBUG
-	qDebug("qtractorSession::trackRecord(\"%s\", %d, %lu)",
-		pTrack->trackName().toUtf8().constData(), int(bRecord), iClipStart);
+	qDebug("qtractorSession::trackRecord(\"%s\", %d, %lu, %lu)",
+		pTrack->trackName().toUtf8().constData(), int(bRecord),
+		iClipStart, iFrameTime);
 #endif
 
 	// Just ditch the in-record clip...
@@ -1474,7 +1497,7 @@ void qtractorSession::trackRecord (
 			createFilePath(pTrack->trackName(),
 				qtractorAudioFileFactory::defaultExt()),
 			qtractorAudioFile::Write);
-		pTrack->setClipRecord(pAudioClip);
+		pTrack->setClipRecord(pAudioClip, iFrameTime);
 		// One-up audio tracks in record mode.
 		++m_iAudioRecord;
 		break;
@@ -1487,7 +1510,7 @@ void qtractorSession::trackRecord (
 			createFilePath(pTrack->trackName(), "mid"),
 			qtractorMidiClip::defaultFormat(),
 			qtractorMidiFile::Write);
-		pTrack->setClipRecord(pMidiClip);
+		pTrack->setClipRecord(pMidiClip, iFrameTime);
 		// MIDI adjust to playing queue start
 		// iif armed while already playing ...
 		if (isPlaying()) {
