@@ -279,13 +279,20 @@ public:
 	void setRecording(bool bRecording);
 	bool isRecording() const;
 
+	// Loop-recording/take mode.
+	void setLoopRecordingMode(int iLoopRecordingMode);
+	int loopRecordingMode() const;
+
+	// Loop-recording/take state.
+	bool isLoopRecording() const;
+
 	// Track recording specifics.
 	unsigned short audioRecord() const;
 	unsigned short midiRecord() const;
 
 	// Special track-immediate methods.
-	void trackRecord(
-		qtractorTrack *pTrack, bool bRecord, unsigned long iClipStart);
+	void trackRecord(qtractorTrack *pTrack, bool bRecord,
+		unsigned long iClipStart, unsigned long iFrameTime);
 	void trackMute(qtractorTrack *pTrack, bool bMute);
 	void trackSolo(qtractorTrack *pTrack, bool bSolo);
 
@@ -435,6 +442,9 @@ private:
 	// Consolidated record state.
 	bool m_bRecording;
 
+	// Loop-recording/take mode.
+	int m_iLoopRecordingMode;
+
 	// Auto time-stretching global flag (when tempo changes)
 	bool m_bAutoTimeStretch;
 
