@@ -24,6 +24,8 @@
 
 #include "qtractorScrollView.h"
 
+#include <QAbstractListModel>
+
 #include <QPixmap>
 
 
@@ -35,6 +37,8 @@ class qtractorTrackButton;
 class qtractorRubberBand;
 class qtractorMidiManager;
 
+class qtractorCurveButton;
+
 class QHeaderView;
 
 class QResizeEvent;
@@ -56,6 +60,8 @@ public:
 		qtractorTrackList *pTrackList, qtractorTrack *pTrack);
 
 	// Local child widgets accessors.
+	qtractorCurveButton *curveButton() const
+		{ return m_pCurveButton; }
 	qtractorTrackButton *recordButton() const
 		{ return m_pRecordButton; }
 	qtractorTrackButton *muteButton() const
@@ -66,6 +72,7 @@ public:
 private:
 
 	// The local child widgets.
+	qtractorCurveButton *m_pCurveButton;
 	qtractorTrackButton *m_pRecordButton;
 	qtractorTrackButton *m_pMuteButton;
 	qtractorTrackButton *m_pSoloButton;
@@ -121,7 +128,8 @@ public:
 	int currentTrackRow() const;
 	int trackRowCount() const;
 
-	// Retrieve current selected track reference.
+	// Current selected track reference.
+	void setCurrentTrack(qtractorTrack *pTrack);
 	qtractorTrack *currentTrack() const;
 
 	// Update the list view item from track pointer reference.
@@ -244,6 +252,9 @@ private:
 
 	qtractorRubberBand *m_pRubberBand;
 
+	// To avoid update reentries.
+	int m_iUpdateContents;
+
 	// Track icon pixmap stuff.
 	enum { IconAudio = 0, IconMidi = 1, IconCount = 2 };
 
@@ -251,6 +262,36 @@ private:
 };
 
 
+//----------------------------------------------------------------------------
+// qtractorTrackListHeaderModel -- Track-list header model.
+
+class qtractorTrackListHeaderModel : public QAbstractListModel
+{
+	Q_OBJECT
+
+public:
+
+	// Constructor.
+	qtractorTrackListHeaderModel(QObject *pParent = 0);
+
+	QVariant headerData(int section, Qt::Orientation orient, int role) const;
+
+	int rowCount(const QModelIndex&) const
+		{ return 0; }
+
+	int columnCount(const QModelIndex&) const
+		{ return m_headerText.count(); }
+
+	QVariant data(const QModelIndex&, int) const
+		{ return QVariant(); }
+
+private:
+
+	// Model variables.
+	QStringList m_headerText;
+};
+
+
 #endif  // __qtractorTrackList_h
 
 
